#! /usr/local/bin/perl

#Download bandwidth limiter
#Script logs all file transfer sizes. When limit is reached, it will
#not allow downloads from the site until the expiration date has passed.
#Useful for ensuring an upload quota is not exceeded.
# *This version passes an http referrer, so a user with a packet sniffer could
#  circumvent this program.
# *This program will not stop a download it if causes you to meet the quota
#  so be sure to leave some headroom.
# *This program uses the syslog facility to drop notice-level messages if
#  failures occur.
#Kamikazi  rev. 8/30/02  www.blastzone.net
# This file may be freely distributed. Distribution in whole or part
# on any media for profit is FORBIDDEN!
# (Including charging for the media itself)
#comments or questions can be sent to zzaxusl0a@hotmail.com 
#or admin@blastzone.net if it works.
#other notes:  The log file must be world-writable (nature of the beast)
#            as always, take necessary precautions.

#set up includes
use CGI;
use Time::Local;
use Sys::Syslog qw( :DEFAULT setlogsock );

##set up local variables
#you will want to change these to fit your site!
#files and locations:
$basedir = "http://localhost/download/"; #the web address to the files
$localdir = "/webdocs/download/"; #the local path to the files
$logfile = "logfile.log";  #the name of the logfile (and lockfile)

#upload quota restriction
$dl_limit = 100000000; #max limit in bytes
$log_life = 30; #logfile life in days

#other tuning variables, you dont need to change these.
$lock_sec = 3; #how long the script should wait if the file is locked
$lockext = ".lock"; #lock file extension (no need to change)

##Here we go!
#convert logfile life into seconds
$log_life = $log_life * 86400;


#obtain file request
$q = new CGI;
$file_request = $q->param('file');

#input error checking here
if (!$file_request)
	{ error_handler("badrequest",0); }

#check for file size and existence	
$filename = $localdir.$file_request;
$file_size = (-s $filename);


if(!$file_size)
	{ error_handler("nodlfile",0); }

#check for log file
open(LOG, $localdir .$logfile) || error_handler("nologfile",0);

#log file opened, check for expiration and current quota size
$line = <LOG>;
$expiration = <LOG>;
chop($line) if $line =~/n$/;
chop($expiration) if $expiration =~/n$/;

#close log file
close(LOG);

#if expriation date is passed, continue download and reset counter
$tm = timelocal($seconds,$minutes,$hours, (localtime)[3,4,5]);
if ($tm >= $expiration)
	{
		#logfile expired, reset counter info
		$line = 0;
		$expiration = ($tm + $log_life);
	}

#if over limit, throw error and pass expiration date
if($line >= $dl_limit)
	{ error_handler("dlmaxxedout",$expiration); }

#check for lock file. if locked, die
if ( !&check_lock($lock_sec) )
	{ error_handler("filelocked",0); }

#update log file
$line = $line + $file_size;
open(LOG, ">$localdir$logfile") || error_handler("updatefailure",0);
print LOG $line."\n";
print LOG $expiration."\n";
close(LOG);

#delete lock file
clean_up();

#everything successful - throw out http referrer
print("Location: $basedir$file_request\n\n");

#we're done

#subroutines follow:
#error_handler
#prints html page describing error
sub error_handler
{
	#open log facility
	#serious problems (server-side) are logged using syslog
	setlogsock('unix');
	openlog("$localdir$logfile",'','user');

	#display HTTP header for error page
	print $q->header();
	print $q->start_html(-title=>'Download Error');
	print '<table border=0 width=100% height=100%>
		<tr><td width=100% height=100% align=center valign=center>
		<font size=+1 color=#ff0000>Download Error</font><br>
		';

	#if-elsif statements to handle error conditions
	if($_[0] eq "badrequest" || $_[0] eq "nodlfile")
		{
		print("404: Not Found<br>
			The file you requested does not exist.");	
		}
	elsif($_[0] eq "dlmaxxedout")
		{
		@newtime = localtime($_[1]);
		$new_month = $newtime[4];
		$new_day = $newtime[3];
		print("The download limit for this server has been reached.<br>
			We apologize for the inconvenience.<br>
			Downloads will be re-enabled on $new_month/$new_day.");
		}
	else
		{
			if($_[0] eq "nologfile")
			{ syslog('notice','Lock File is missing'); }

			elsif($_[0] eq "filelocked")
			{ syslog('notice','Lock File is stuck'); }
			
			elsif($_[0] eq "updatefailure")
			{ syslog('notice','Logfile is damaged'); }

			else
			{ syslog('notice',"Unknown error: $_[0]"); }

			print('A server-side error has occurred.<br>
			The administrator has been notified,<br>
			please try again later.');
		}
	print '</td></tr></table>
		</body></html>';

	closelog();
	exit;
}

#checklock
#based on a lock function by Matthew M. Wright
#returns 0 if lock file not created
#returns 1 if lock file successfully created
sub check_lock
{
	$time = $_[0];
	
	for($i = 1; $i <= $time; $i++)
	{
		if(-e "$localdir$logfile$lockext" )
		{
			sleep 1;
		}
		else
		{
			open(LOCK,">$localdir$logfile$lockext");
			print LOCK "0";
			close(LOCK);
			return 1;
		}
	}
	return 0;
}

#clean up
#based on a function by Matthew M. Wright
#kills the lock file
sub clean_up
{
	unlink("$localdir$logfile$lockext");
}
#EOF
